import getopt
import os
import shutil
import sys
from urllib.parse import urlencode

import pyperclip

from .parse_curl import *


def key2hump(key):
    """
    try to capitalize the underscore
    :param key:
    :type key:
    :return:
    :rtype:
    """
    return key.title().replace("_", "")


def parse_curlstring(filestring):
    """
    curl to requests code
    :param filestring:
    :type filestring:
    :return:
    :rtype:
    """
    curl_cmd = parse_curl_command(filestring)
    output = """# -*- coding: utf-8 -*-
\"\"\"
kkba generates this file
kkba doucment: https://github.com/kaikeba/kkba
\"\"\"
import requests\n
"""
    req = ['response = requests.{}("{}"'.format(curl_cmd.method, curl_cmd.url)]
    if curl_cmd.params:
        output += "params = {}\n\n".format(prettier_tuple(curl_cmd.params))
        req.append('params=params')
    if curl_cmd.data:
        if isinstance(curl_cmd.data, dict):
            output += "data = {}\n\n".format(prettier_dict(curl_cmd.data))
        elif isinstance(curl_cmd.data, str):
            output += "data = '{}'\n\n".format(curl_cmd.data)
        else:
            output = 'from requests_toolbelt import MultipartEncoder\n' + output
            output += "data = {}\n\n".format(format_multi(curl_cmd.data))
        req.append('data=data')
    if curl_cmd.headers:
        output += "headers = {}\n\n".format(prettier_dict(curl_cmd.headers))
        req.append('headers=headers')
    if curl_cmd.cookies:
        output += "cookies = {}\n\n".format(prettier_dict(curl_cmd.cookies))
        req.append('cookies=cookies')
    if curl_cmd.insecure:
        req.append('verify=False')
    output += ', '.join(req) + ')\n\n'
    output += 'print(response.text)\n\n'
    url = curl_cmd.url
    filename = re.findall('://(www\.)?(.*?)\.', url)
    filename = filename[0]
    if filename:
        filename = filename[1].replace('.', '_').replace('www_', '')
    else:
        filename = 'test.py'

    filename = filename + '.py'
    return output, filename


def parse_curl_string_feapder(filestring):
    """
    curl to feapder synchronize code
    :param filestring:
    :type filestring:
    :return:
    :rtype:
    """
    curl_cmd = parse_curl_command(filestring)
    output = """# -*- coding: utf-8 -*-
\"\"\"
kkba generates this file
feapder document： http://boris.org.cn/feapder/#/README
kkba doucment: https://github.com/kaikeba/kkba
\"\"\"

from feapder import Request
import feapder.setting as setting

setting.LOG_LEVEL = 'ERROR'


def start():
"""
    req = ['    request = Request(url="{}"'.format(curl_cmd.url)]

    if curl_cmd.params:
        params_str = '(\n' + " " * 8 + ("," + "\n" + " " * 8).join(
            str(i) for i in curl_cmd.params) + ',\n    )'
        output += "    params = {}\n\n".format(params_str)
        req.append('params=params')
    if curl_cmd.data:
        if isinstance(curl_cmd.data, dict):
            output += "    data = {}\n\n".format(prettier_dict(curl_cmd.data))
        elif isinstance(curl_cmd.data, str):
            output += "    data = '{}'\n\n".format(curl_cmd.data)
        else:
            output = 'from requests_toolbelt import MultipartEncoder\n' + output
            output += "    data = {}\n\n".format(format_multi(curl_cmd.data))
        req.append('data=data')
    if curl_cmd.headers:
        output += "    headers = {}\n\n".format(prettier_dict(curl_cmd.headers))
        req.append('headers=headers')
    if curl_cmd.cookies:
        output += "    cookies = {}\n\n".format(prettier_dict(curl_cmd.cookies))
        req.append('cookies=cookies')
    if curl_cmd.insecure:
        req.append('verify=False')
    output += ', '.join(req) + ')\n'
    output += """
    response = request.get_response()
    if response.ok:
        return response
    else:
        return 'error'


if __name__ == '__main__':
    res = start()
    print(res.text)
"""
    url = curl_cmd.url
    filename = re.findall('://(www\.)?(.*?)\.', url)
    filename = filename[0]
    if filename:
        filename = filename[1].replace('.', '_').replace('www_', '')
    else:
        filename = 'test.py'
    filename = filename + '.py'
    return output, filename


def fetch_curl(curl_args):
    """
    parse and fetch curl request
    :param curl_args:
    [url, '-H', 'xxx', '-H', 'xxx', '--data-binary', '{"xxx":"xxx"}', '--compressed']
    :return:
    """
    url = curl_args[0]
    curl_args.pop(0)

    headers = {}
    data = {}
    for i in range(0, len(curl_args), 2):
        if curl_args[i] == "-H":
            regex = "([^:\s]*)[:|\s]*(.*)"
            result = re.search(regex, curl_args[i + 1], re.S).groups()
            if result[0] in headers:
                headers[result[0]] = headers[result[0]] + "&" + result[1]
            else:
                headers[result[0]] = result[1].strip()

        elif curl_args[i] == "--data-binary":
            data = json.loads(curl_args[i + 1])

    return url, data, headers


def generate_feapder_air_spider(clip_curl):
    """
    curl to feapder AirSpider code
    :param clip_curl:
    :type clip_curl:
    :return:
    :rtype:
    """
    curl_cmd = parse_curl_command(clip_curl)
    template_path = os.path.abspath(
        os.path.join(__file__, "../", 'feapder_air_spider_template.tmpl')
    )
    with open(template_path, 'r', encoding='utf-8') as file:
        file_str = file.read()
    req = ['        yield feapder.Request(url="{}"'.format(curl_cmd.url)]
    output = ''
    output_p = ''
    if curl_cmd.params:
        params_str = '(\n' + " " * 12 + ("," + "\n" + " " * 12).join(
            str(i) for i in curl_cmd.params) + ',\n        )'
        output += "        params = {}\n\n".format(params_str)
        req.append('params=params')
    if curl_cmd.data:
        if isinstance(curl_cmd.data, dict):
            output += "        data = {}\n\n".format(prettier_dict(curl_cmd.data, indent=8))
        elif isinstance(curl_cmd.data, str):
            output += "        data = '{}'\n\n".format(curl_cmd.data)
        else:
            output = 'from requests_toolbelt import MultipartEncoder\n' + output
            output += "    data = {}\n\n".format(format_multi(curl_cmd.data, indent=8))
        req.append('data=data')
    if curl_cmd.headers:
        output_p += "        request.headers = {}\n\n".format(prettier_dict(curl_cmd.headers, indent=8))
    if curl_cmd.cookies:
        output_p += "        request.cookies = {}\n".format(prettier_dict(curl_cmd.cookies, indent=8))
    if curl_cmd.insecure:
        req.append('verify=False')
    output += ', '.join(req) + ')\n'
    file_str = file_str.replace('${params}', output)
    file_str = file_str.replace('${request}', output_p)
    url = curl_cmd.url
    filename = re.findall('://(www\.)?(.*?)\.', url)
    filename = filename[0]
    if filename:
        filename = filename[1].replace('.', '_').replace('www_', '')
    else:
        filename = 'kaikeba'
    if filename.islower():
        spider_name = key2hump(filename)
    file_str = file_str.replace("${spider_name}", spider_name).replace("${spider_name_l}", filename)
    filename = filename + '.py'
    return file_str, filename


def generate_single_scrapy(clip_curl):
    """
    curl to scrapy single file code
    :param clip_curl:
    :type clip_curl:
    :return:
    :rtype:
    """
    curl_cmd = parse_curl_command(clip_curl)
    template_path = os.path.abspath(
        os.path.join(__file__, "../", 'scrapy_template.tmpl')
    )
    with open(template_path, 'r', encoding='utf-8') as file:
        file_str = file.read()
    req = ['        yield scrapy.Request(url="{}"'.format(curl_cmd.url)]
    output = ''
    if curl_cmd.params:
        url_ = urlencode(curl_cmd.params)
        url__ = curl_cmd.url + '?' + url_
        req = ['        yield scrapy.Request(url="{}"'.format(url__)]
    if curl_cmd.data:
        if isinstance(curl_cmd.data, dict):
            output += "        data = {}\n\n".format(prettier_dict(curl_cmd.data, indent=8))
            req[0] = req[0].replace('Request', 'FormRequest')
            req.append('formdata=data')
        elif isinstance(curl_cmd.data, str):
            output += "        data = '{}'\n\n".format(curl_cmd.data)
            req.append('method="POST", body=data')
        else:
            output = 'from requests_toolbelt import MultipartEncoder\n' + output
            output += "    data = {}\n\n".format(format_multi(curl_cmd.data, indent=8))
            req[0] = req[0].replace('Request', 'FormRequest')
            req.append('formdata=data')
    if curl_cmd.headers:
        output += "        headers = {}\n\n".format(prettier_dict(curl_cmd.headers, indent=8))
        req.append('headers=headers')
    if curl_cmd.cookies:
        output += "        cookies = {}\n".format(prettier_dict(curl_cmd.cookies, indent=8))
        req.append('cookies=cookies')
    if curl_cmd.insecure:
        req.append('verify=False')
    output += ', '.join(req) + ', callback=self.parse)\n'
    file_str = file_str.replace('${params}', output)
    url = curl_cmd.url

    filename = re.findall('://(www\.)?(.*?)\.', url)
    filename = filename[0]
    if filename:
        filename = filename[1].replace('.', '_').replace('www_', '')
    else:
        filename = 'kaikeba'
    if filename.islower():
        spider_name = key2hump(filename)
    file_str = file_str.replace("${spider_name}", spider_name).replace("${spider_name_l}", filename)
    return file_str, filename


def create_readme():
    """
    create readme.md
    :return:
    :rtype:
    """
    old_path = os.path.abspath(
        os.path.join(__file__, '../', 'README.md.tmpl')
    )
    new_filename = 'README.md'
    while os.path.exists(new_filename):
        filename_md_ = input("There is'README.md' in the current directory. If you overwrite this file, please press "
                             "Enter or enter a new file name:")
        if not filename_md_:
            print("输入的空字符，生成文件覆盖源文件")
            break
        else:
            new_filename = filename_md_ + '.md'
    return old_path, new_filename


def usage():
    """
爬虫生成器

usage: kkba [options]

optional arguments:
  -F,               推荐: 将粘贴板curl或者url，生成feapder异步爬虫代码，相当于scrapy的用法
  -s                将粘贴板curl或者url，生成scrapy单文件项目
  -f,               将粘贴板curl或者url，生成feapder同步爬虫代码，相当于requests的用法
  -r,               将粘贴板curl或者url，生成requests爬虫代码
  -h, --help        帮助文档
  -v, --version     查看版本
    """
    print(usage.__doc__)
    sys.exit()


def main_cmd():
    args = sys.argv

    try:  # no params
        if args[1] in ("-h", "--help"):
            usage()

        elif args[1] in ("-v", "-V", "--version"):
            version_path = os.path.abspath(
                os.path.join(__file__, '../', 'VERSION')
            )
            with open(version_path, 'r') as f:
                version_str = f.read()
                print(version_str)
            sys.exit()
    except IndexError:
        usage()


def create_chr(dirname):
    """
    create directory path
    :param dirname:
    :type dirname:
    :return:
    :rtype:
    """
    dirname = dirname.replace('.py', '')
    while os.path.exists(dirname):
        dirname_ = input(
            f"Pathname {dirname} conflict, if you overwrite this directory, please enter Enter or enter a new path "
            "name:")
        if not dirname_:
            print("Input enter to generate a file to overwrite the source file")
            shutil.rmtree(dirname)
            break
        else:
            dirname = dirname_
    os.makedirs(dirname)
    os.chdir(dirname)
    return dirname


def get_chr(filename, output):
    """
    get directory path and spiders file
    :param filename:
    :type filename:
    :param output:
    :type output:
    :return:
    :rtype:
    """
    dirname = create_chr(filename)
    while os.path.exists(filename):
        filename_ = input(
            f"Pathname {filename} conflict, if you want overwrite this directory, please enter Enter or "
            f"enter a new path name:")
        if not filename_:
            print("Input enter to generate a file to overwrite the source file")
            break
        else:
            filename = filename_ + '.py'

    # check if exists readme.md
    old_path, new_filename = create_readme()
    shutil.copyfile(old_path, new_filename)
    open('./' + filename, 'w', encoding='utf-8').write(output)
    return 'green', f'Convertion Finished.\nPlease open {dirname} to check the code.'


def convert_main(opt):
    """
    According to the command parameter convert code
    :param opt:
    :type opt:
    :return:
    :rtype:
    """
    main_cmd()
    result_type = {
        0: 'Convertion Finished.\nNow requests code is copyed in your clipboard.',
        1: 'Please copy the curl or url before run.',
        2: 'Usage:\n\tCurl File Convert -- kkba -F requests.curl\n\tClipboard Convert -- kkba',
    }
    if opt:
        if ('-f' in opt[0]) or ('-F' in opt[0]):
            clip = str(pyperclip.paste())
            if clip.find('curl ') == 0:
                if '-f' in opt[0]:
                    output, filename = parse_curl_string_feapder(clip)
                elif '-F' in opt[0]:
                    output, filename = generate_feapder_air_spider(clip)
                return get_chr(filename, output)
            elif clip.find('http') == 0:
                clip = 'curl ' + clip
                if '-f' in opt[0]:
                    output, filename = parse_curl_string_feapder(clip)
                elif '-F' in opt[0]:
                    output, filename = generate_feapder_air_spider(clip)
                return get_chr(filename, output)
            else:
                return 'yellow', result_type[1]
            return 'red', result_type[2]
        elif '-s' in opt[0]:
            clip = str(pyperclip.paste())
            if clip.find('curl ') == 0:
                output, filename = generate_single_scrapy(clip)
                return get_chr(filename, output)
            elif clip.find('http') == 0:
                clip = 'curl ' + clip
                output, filename = generate_single_scrapy(clip)
                return get_chr(filename, output)
            else:
                return 'yellow', result_type[1]
            return 'red', result_type[2]

        elif '-r' in opt[0]:
            clip = str(pyperclip.paste())
            if clip.find('curl ') == 0:
                output, filename = parse_curlstring(clip)
                return get_chr(filename, output)
            elif clip.find('http') == 0:
                clip = 'curl ' + clip
                output, filename = parse_curlstring(clip)
                return get_chr(filename, output)
            else:
                return 'yellow', result_type[1]
            return 'red', result_type[2]

        else:
            usage()
    else:
        usage()


def main():
    """
    main function
    :return:
    :rtype:
    """
    _opts = ['version', 'help']
    _s_opts = 'rfFsSvVh'
    opt, arg = getopt.getopt(sys.argv[1:], _s_opts, _opts)
    color, feed_back = convert_main(opt)
    console = Console()
    console.print(f'=' * os.get_terminal_size()[0], justify='center')
    console.print(f'[bold {color}]{feed_back}[/]', justify='center')
    console.print(f'=' * os.get_terminal_size()[0], justify='center')



import base64
import logging

import requests
from requests.auth import HTTPBasicAuth
from w3lib.http import basic_auth_header


class Proxy(object):
    def __init__(
            self,
            crawlerType='requests',
            proxyType=None,
            username=None,
            password=None,
            proxy_url=None,
            **kwargs
    ):
        """
        :param crawlerType: crawler type, value is requests or scrapy
        :type crawlerType:
        :param proxyType: proxy type,  value is abuyun、kuaidaili、qingting
        :type proxyType:
        :param username:
        :type username:
        :param password:
        :type password:
        :param proxy_url:
        :type proxy_url:
        :param kwargs:
        :type kwargs:
        """
        self.crawlerType = crawlerType
        self.proxyType = proxyType
        self.username = username
        self.password = password
        self.proxy_url = proxy_url

    def get_proxy(self, request=None):
        if self.proxyType and self.username and self.password:
            proxyType = self.proxyType
        else:
            logging.error('Please input proxy type, proxy username and password, proxy server api')
            return
        if self.crawlerType == 'requests':
            if proxyType == 'qingting':
                return self.__qingting_requests()
            if proxyType == 'kuaidaili':
                return self.__kuaidaili_requests()
            elif proxyType == 'abuyun':
                return self.__abuyun_request()
            else:
                logging.error('No proxy type of {}'.format(proxyType))
        if self.crawlerType == 'scrapy':
            if not request:
                logging.error('Scrapy must params request object')
                return
            if proxyType == 'qingting':
                return self.__qingting_scrapy(request)
            elif proxyType == 'kuaidaili':
                return self.__kuaidaili_scrapy(request)
            elif proxyType == 'abuyun':
                return self.__abuyun_scrapy(request)
            else:
                logging.error('No proxy type of {}'.format(proxyType))
        else:
            logging.error('No support this proxy type')

    def __qingting_requests(self):
        """
        qingting proxy requests
        :return:
        :rtype:
        """
        if not self.proxy_url:
            logging.error('qingting proxy need input proxy api')
            return
        proxy_text = requests.get(self.proxy_url, auth=HTTPBasicAuth(self.username, self.password)).text.strip()
        proxies = {
            'http': proxy_text,
            'https': proxy_text
        }
        return proxies

    def __qingting_scrapy(self, request):
        """
        qingting proxy scrapy
        :param request:
        :type request:
        :return:
        :rtype:
        """
        if not self.proxy_url:
            logging.error('qingting proxy need input proxy api')
            return
        proxy_text = requests.get(self.proxy_url, auth=HTTPBasicAuth(self.username, self.password)).text.strip()
        request.meta['proxy'] = "http://" + proxy_text
        return request

    def __kuaidaili_requests(self):
        """
        kuai proxy requests
        :return:
        :rtype:
        """
        if not self.proxy_url:
            self.proxy_url = "tps163.kdlapi.com:15818"
        tunnel = self.proxy_url

        # username and password authration
        proxies = {
            "http": "http://%(user)s:%(pwd)s@%(proxy)s/" % {"user": self.username, "pwd": self.password,
                                                            "proxy": tunnel},
            "https": "http://%(user)s:%(pwd)s@%(proxy)s/" % {"user": self.username, "pwd": self.password,
                                                             "proxy": tunnel}
        }
        return proxies

    def __kuaidaili_scrapy(self, request):
        """
        kuai proxy scrapy
        :param request:
        :type request:
        :return:
        :rtype:
        """
        if not self.proxy_url:
            self.proxy_url = "tps163.kdlapi.com:15818"
        proxy = self.proxy_url
        request.meta['proxy'] = "http://%(proxy)s" % {'proxy': proxy}
        # username and password authration
        # Whitelist authentication can comment this line
        request.headers['Proxy-Authorization'] = basic_auth_header(self.username, self.password)
        request.headers["Connection"] = "close"
        return request

    def __abuyun_request(self):
        """
        abuhun request
        :return:
        :rtype:
        """
        if not self.proxy_url:
            self.proxy_url = "http-dyn.abuyun.com"
        proxyHost = self.proxy_url
        proxyPort = "9020"

        proxyMeta = "http://%(user)s:%(pass)s@%(host)s:%(port)s" % {
            "host": proxyHost,
            "port": proxyPort,
            "user": self.username,
            "pass": self.password,
        }

        proxies = {
            "http": proxyMeta,
            "https": proxyMeta,
        }
        return proxies

    def __abuyun_scrapy(self, request):
        """
        abuyun scrapy
        :param request:
        :type request:
        :return:
        :rtype:
        """
        if not self.proxy_url:
            self.proxy_url = "http://http-dyn.abuyun.com:9020"
        proxyServer = self.proxy_url
        # for Python2
        proxyAuth = "Basic " + base64.urlsafe_b64encode(bytes((self.username + ":" + self.password), "ascii")).decode(
            "utf8")
        request.meta["proxy"] = proxyServer

        request.headers["Proxy-Authorization"] = proxyAuth
        return request


if __name__ == '__main__':
    p = Proxy(crawlerType='requests', proxyType='xxx', username='xxx', password='xxx')
    proxy = p.get_proxy()
    url = 'http://httpbin.org/get'
    res = requests.get(url, proxies=proxy)
    print(res.text)



import getopt

from .ulti import *


def main():
    _opts = ['help', 'version']
    _s_opts = 'rfFsSvVh'
    opt, arg = getopt.getopt(sys.argv[1:], _s_opts, _opts)
    color, feed_back = convert_main(opt)
    console = Console()
    console.print(f'=' * os.get_terminal_size()[0], justify='center')
    console.print(f'[bold {color}]{feed_back}[/]', justify='center')
    console.print(f'=' * os.get_terminal_size()[0], justify='center')


if __name__ == '__main__':
    main()



import argparse
import json
import re
import shlex
from collections import OrderedDict
from os import get_terminal_size
from urllib.parse import urlparse, unquote

from rich.console import Console
from rich.syntax import Syntax


def prettier_print(code: str):
    """
    pretter print
    :param code:
    :type code:
    :return:
    :rtype:
    """
    syntax = Syntax(code, "python", theme="monokai", line_numbers=True)
    console = Console()
    console_width = int((get_terminal_size()[0] - 36) / 2)
    console.print("=" * console_width +
                  "[bold magenta] Python Requests Code Preview Start [/]" +
                  "=" * console_width,
                  justify='center')
    console.print(syntax)
    console.print("=" * console_width +
                  "[bold magenta]  Python Requests Code Preview End  [/]" +
                  "=" * console_width,
                  justify='center')


def parse_content_type(content_type: str):
    """
    parse content type
    :param content_type:
    :type content_type:
    :return:
    :rtype:
    """
    parts = content_type.split(';', 1)
    tuparts = parts[0].split('/', 1)
    if len(tuparts) != 2:
        return None
    dparts = OrderedDict()
    if len(parts) == 2:
        for i in parts[1].split(";"):
            c = i.split("=", 1)
            if len(c) == 2:
                dparts[c[0].strip()] = c[1].strip()
    return tuparts[0].lower(), tuparts[1].lower(), dparts


def format_multi(the_multi_list, indent=4):
    """
    format multi data
    :param the_multi_list:
    :type the_multi_list:
    :param indent:
    :type indent:
    :return:
    :rtype:
    """
    return 'MultipartEncoder(\n' + " " * indent + 'fields=[\n' + " " * indent * 2 + (
            ",\n" + " " * indent * 2).join(map(str, the_multi_list)) + '\n])'


def parse_multi(content_type, the_data):
    """
    parse multi data
    :param content_type:
    :type content_type:
    :param the_data:
    :type the_data:
    :return:
    :rtype:
    """
    boundary = b''
    if content_type:
        ct = parse_content_type(content_type)
        if not ct:
            print('nocontent-type')
            return ['no content-type']
        try:
            boundary = ct[2]["boundary"].encode("ascii")
        except (KeyError, UnicodeError):
            print('no boundary')
            return ['no boundary']
    if boundary:
        result = []
        for i in the_data.split(b"--" + boundary):
            p = i.replace(b'\\x0d', b'\r')
            p = p.replace(b'\\x0a', b'\n')
            p = p.replace(b'\\n', b'\n')
            p = p.replace(b'\\r', b'\r')
            parts = p.splitlines()
            print(parts)
            if len(parts) > 1 and parts[0][0:2] != b"--":
                if len(parts) > 4:
                    tmp_value = {}
                    key, tmp_value['filename'] = re.findall(
                        br'\bname="([^"]+)"[^"]*filename="([^"]*)"',
                        parts[1])[0]
                    tmp_value['content'] = b"".join(
                        parts[3 + parts[2:].index(b""):])
                    tmp_value['content_type'] = parts[2]
                    value = (tmp_value['filename'].decode(),
                             tmp_value['content'].decode(),
                             tmp_value['content_type'].decode())
                else:
                    key = re.findall(br'\bname="([^"]+)"', parts[1])[0]
                    value = (b"".join(parts[3 +
                                            parts[2:].index(b""):])).decode()
                result.append((key.decode(), value))
        return result


def parse_args(curl_cmd):
    """
    argparse parse curl params
    :param curl_cmd:
    :type curl_cmd:
    :return:
    :rtype:
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('command')
    parser.add_argument('url')
    parser.add_argument('-d', '--data')
    parser.add_argument('-b', '--cookie', default=None)
    parser.add_argument('--data-binary',
                        '--data-raw',
                        '--data-ascii',
                        default=None)
    parser.add_argument('-X', default='')
    parser.add_argument('-F', '--form', default=None)
    parser.add_argument('-H', '--header', action='append', default=[])
    parser.add_argument('-A', '--user-agent', default='')
    parser.add_argument('--compressed', action='store_true')
    parser.add_argument('-k', '--insecure', action='store_true')
    parser.add_argument('-I', '--head', action='store_true')
    parser.add_argument('-G', '--get', action='store_true')
    parser.add_argument('--user', '-u', default=())
    parser.add_argument('-i', '--include', action='store_true')
    parser.add_argument('-s', '--silent', action='store_true')
    cmd_set = shlex.split(curl_cmd)
    arguments = parser.parse_args(cmd_set)
    return arguments


def prettier_dict(the_dict, indent=4):
    """
    pretter dict
    :param the_dict:
    :type the_dict:
    :param indent:
    :type indent:
    :return:
    :rtype:
    """
    if not the_dict:
        return "{}"
    return ("\n" + " " * indent).join(
        json.dumps(the_dict,
                   sort_keys=True,
                   indent=indent,
                   separators=(',', ': ')).splitlines())


def prettier_tuple(the_tuple, indent=4):
    """
    pretter tuple
    :param the_tuple:
    :type the_tuple:
    :param indent:
    :type indent:
    :return:
    :rtype:
    """
    if not the_tuple:
        return "()"
    return '(\n' + " " * indent + ("," + "\n" + " " * indent).join(
        str(i) for i in the_tuple) + ',\n)'


def curl_replace(curl_cmd):
    """
    replace curl part string
    :param curl_cmd:
    :type curl_cmd:
    :return:
    :rtype:
    """
    curl_str_replace = [(r'\\\r|\\\n|\r|\n', ''), (' -XPOST', ' -X POST'),
                        (' -XGET', ' -X GET'), (' -XPUT', ' -X PUT'),
                        (' -XPATCH', ' -X PATCH'), (' -XDELETE', ' -X DELETE'),
                        (' -Xnull', '')]
    tmp_curl_cmd = curl_cmd
    for pattern in curl_str_replace:
        tmp_curl_cmd = re.sub(pattern[0], pattern[1], tmp_curl_cmd)
    return tmp_curl_cmd.strip()


class parse_curl_command:
    def __init__(self, curl_cmd):
        """
        parse curl command
        :param curl_cmd: curl command
        :type curl_cmd: str
        """
        self.curl_cmd = curl_replace(curl_cmd)
        self.arguments = parse_args(self.curl_cmd)
        self.method = 'get'
        post_data = self.arguments.data or self.arguments.data_binary
        self.urlparse = urlparse(self.arguments.url)
        self.url = "{}://{}{}".format(self.urlparse.scheme,
                                      self.urlparse.netloc, self.urlparse.path)
        self.cookies = None
        if self.urlparse.query:
            self.params = tuple(
                re.findall(r'([^=&]*)=([^&]*)', unquote(self.urlparse.query)))
        else:
            self.params = ()
        headers = self.arguments.header
        cookie_string = ''
        content_type = ''
        if headers:
            self.headers = dict(
                [tuple(header.split(': ', 1)) for header in headers])
            cookie_string = self.headers.get('cookie') or self.headers.get(
                'Cookie')
            if 'cookie' in self.headers:
                self.headers.pop('cookie')
            if 'Cookie' in self.headers:
                self.headers.pop('Cookie')
            content_type = self.headers.get(
                'Content-Type') or self.headers.get('content-type')
        else:
            self.headers = {}
        if self.arguments.cookie:
            cookie_string = self.arguments.cookie
        if post_data and not self.arguments.get:
            self.method = 'post'
            if "multipart/form-data" in content_type.lower():
                self.data = parse_multi(
                    content_type,
                    unquote(post_data.strip('$')).encode('raw_unicode_escape'))
            elif '{"' in post_data:
                self.data = post_data
            else:
                self.data = dict(
                    re.findall(r'([^=&]*)=([^&]*)', unquote(post_data)))
        elif post_data:
            self.params = tuple(
                re.findall(r'([^=&]*)=([^&]*)', unquote(post_data)))
            self.data = {}
        else:
            self.data = {}
        if self.arguments.X:
            self.method = self.arguments.X.lower()
        if cookie_string:
            self.cookies = dict(
                re.findall(r' ([^=\s]*)=([^;]*)', cookie_string))
        if self.arguments.insecure:
            self.insecure = True
        else:
            self.insecure = False
